# -*- coding: utf-8 -*-
"""VideoHD Login dialog."""
import xbmc
import xbmcgui
import xbmcaddon

from lib.static import *

from lib import g, ID, ACTION

__addon__ = xbmcaddon.Addon()
__addonpath__ = __addon__.getAddonInfo('path').decode('utf-8')
__language__ = __addon__.getLocalizedString


class LoginDialog(xbmcgui.WindowXMLDialog):

    """Login Dialog."""

    def __init__(self, strXMLname, strFallbackPath, strDefaultName, forceFallback=0):
        """Init function."""
        self.loginCancel = False
        self.nextFocusButton = ['HostIP', 'User', 'Passwd', 'HostPort']
        self.rememberMsg = dict()

    def _initLoginMsg(self):
        '''VideoHD first time'''
        for key in self.rememberMsg.keys():
            if self.rememberMsg.get(key):
                initButton = 'Login.{}.Button'.format(key)
                g.clearHomeProperty('Login.Empty.{}'.format(key))

                if key in ('RememberMe', 'EnableSSL', ):
                    initButton = 'Login.{}.Radiobutton'.format(key)
                    if self.rememberMsg.get(key):
                        self.getControl(ID.val(initButton)).setSelected(True)
                elif key in ('Passwd', ):
                    self.getControl(ID.val(initButton)).setLabel(len(self.rememberMsg.get(key)) * PASSWORD_SYMBOL)
                else:
                    self.getControl(ID.val(initButton)).setLabel(self.rememberMsg.get(key))
            else:
                # Set default parameter.
                if key in ('User', ):
                    self.getControl(ID.val('Login.User.Button')).setLabel(DEFAULT_USER)
                    self.rememberMsg['User'] = DEFAULT_USER
                elif key in ('HostPort', ):
                    defaultPort = DEFAULT_SSL_PORT if self.rememberMsg.get('EnableSSL') else DEFAULT_HTTP_PORT
                    self.getControl(ID.val('Login.HostPort.Button')).setLabel(defaultPort)
                    self.rememberMsg['HostPort'] = defaultPort
                else:
                    g.setHomeProperty('Login.Empty.{}'.format(key), 'true')

    def onInit(self):
        """onInit function."""
        g.setWaitingMask(self, state='init')
        self.rememberMsg = g.getVideoHDConfValue()
        self._initLoginMsg()

    def _focusHandler(self, currentFocus):
        nextFocusIdx = self.nextFocusButton.index(currentFocus) + 1

        if nextFocusIdx in (len(self.nextFocusButton), ):
            return

        nextButton = self.nextFocusButton[nextFocusIdx]
        self.setFocusId(ID.val('Login.{}.Button'.format(nextButton)))

    def onAction(self, action):
        """Action function."""
        if action in (ACTION.val('PREVIOUS_MENU'), ACTION.val('BACK'), ACTION.val('MOUSE_RIGHT_CLICK'), ):
            self.loginCancel = True
            self.close()

    def onClick(self, controlID):
        """Click trigger function."""
        if controlID in (ID.val('Login.RememberMe.Radiobutton'),
                         ID.val('Login.EnableSSL.Radiobutton'), ):
            changeKey = ID.key(controlID).split('.')[1]

            if self.getControl(controlID).isSelected():
                self.rememberMsg[changeKey] = True

                if changeKey in ('EnableSSL', ):
                    self.rememberMsg['HostPort'] = DEFAULT_SSL_PORT
                    self.getControl(ID.val('Login.HostPort.Button')).setLabel(DEFAULT_SSL_PORT)
            else:
                self.rememberMsg[changeKey] = False

                if changeKey in ('EnableSSL', ):
                    self.rememberMsg['HostPort'] = DEFAULT_HTTP_PORT
                    self.getControl(ID.val('Login.HostPort.Button')).setLabel(DEFAULT_HTTP_PORT)

            return

        if controlID in (ID.val('Login.LoginCheck.Button'), ):
            g.setWaitingMask(self, state='on')
            check = g.loginCheck(self.rememberMsg)
            if check.get('result'):
                '''init Global Login Message for setting back to main page'''
                for key in self.rememberMsg.keys():
                    globalLoginProperty = 'Global.Login.{}'.format(key)
                    g.setHomeProperty(globalLoginProperty, g.ensureUTF8(self.rememberMsg.get(key)))

                '''update videohd.conf'''
                g.updateVideoHDConf(self.rememberMsg)

                '''check localhost or not here'''
                # g.checkLocalHost()
                self.close()
            else:
                '''clear Passwd videohd.conf'''
                for key in self.rememberMsg.keys():
                    globalLoginProperty = 'Global.Login.{}'.format(key)
                    g.clearHomeProperty(globalLoginProperty)

                self.rememberMsg['Passwd'] = ''
                g.updateVideoHDConf(self.rememberMsg)

                g.setWaitingMask(self, state='off')
                if not check.get('msg'):
                    return

                if check.get('msg'):
                    g.showDialog('ok', check.get('msg'))

            return

        if controlID in (ID.val('Login.Cancel.Button'), ):
            self.loginCancel = True
            self.close()
            return

        if controlID in ID.groupID('Login.'):
            loginMsgName = ID.key(controlID).split('.')[1]
            control = self.getControl(controlID)

            hidden = True if loginMsgName in ('Passwd', ) else False

            rememberMe = eval(self.rememberMsg.get('RememberMe')) if type(self.rememberMsg.get('RememberMe')) in (str, ) else self.rememberMsg.get('RememberMe')
            # defaultValue = self.rememberMsg.get(loginMsgName) if rememberMe else ''
            defaultValue = self.rememberMsg.get(loginMsgName)

            kb = xbmc.Keyboard(defaultValue,
                               u'Input your message',
                               hidden)
            kb.doModal()

            if kb.isConfirmed():
                kbText = kb.getText()
                if kbText:
                    g.clearHomeProperty('Login.Empty.{}'.format(loginMsgName))
                    if loginMsgName in ('Passwd', ):
                        control.setLabel(PASSWORD_SYMBOL * len(kbText))
                    else:
                        control.setLabel(kbText)
                else:
                    control.setLabel(' ')
                    g.setHomeProperty('Login.Empty.{}'.format(loginMsgName), 'true')

                self.rememberMsg[loginMsgName] = kbText
                self._focusHandler(loginMsgName)

    def close(self):
        """Close function."""
        g.setWaitingMask(self, state='off')
        for item in self.nextFocusButton:
            g.clearHomeProperty('Login.Empty.{}'.format(item))

        super(LoginDialog, self).close()
